import { jsx } from '@emotion/core';
import { createContext, useContext } from 'react';
import { ThemeProvider } from 'emotion-theming';
import { MDXProvider } from '@mdx-js/react';
import merge from 'lodash.merge';
import get from 'lodash.get';
import styled from '@emotion/styled';
import { space, width, color, flex, alignItems, justifyContent, flexWrap, flexDirection } from 'styled-system';
import css from '@styled-system/css';
export { css } from '@styled-system/css';

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function _objectSpread(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};
    var ownKeys = Object.keys(source);

    if (typeof Object.getOwnPropertySymbols === 'function') {
      ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) {
        return Object.getOwnPropertyDescriptor(source, sym).enumerable;
      }));
    }

    ownKeys.forEach(function (key) {
      _defineProperty(target, key, source[key]);
    });
  }

  return target;
}

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

function _objectWithoutProperties(source, excluded) {
  if (source == null) return {};

  var target = _objectWithoutPropertiesLoose(source, excluded);

  var key, i;

  if (Object.getOwnPropertySymbols) {
    var sourceSymbolKeys = Object.getOwnPropertySymbols(source);

    for (i = 0; i < sourceSymbolKeys.length; i++) {
      key = sourceSymbolKeys[i];
      if (excluded.indexOf(key) >= 0) continue;
      if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue;
      target[key] = source[key];
    }
  }

  return target;
}

var jsx$1 = function jsx$$1(type, props) {
  var arguments$1 = arguments;

  for (var _len = arguments.length, children = new Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
    children[_key - 2] = arguments$1[_key];
  }

  return jsx.apply(undefined, [type, props ? _objectSpread({}, props, {
    css: css(props.css)
  }) : null].concat(children));
};

var themed = function themed(key) {
  return function (props) {
    return css(get(props.theme, "styles.".concat(key)))(props.theme);
  };
};

var tags = ['p', 'a', 'h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'img', 'pre', 'code', 'ol', 'ul', 'li', 'blockquote', 'hr', 'em', 'table', 'tr', 'th', 'td', 'em', 'strong', 'delete', // mdx
'inlineCode', 'thematicBreak', // other
'div', // theme-ui
'root'];
var aliases = {
  inlineCode: 'code',
  thematicBreak: 'hr',
  root: 'div'
};

var alias = function alias(n) {
  return aliases[n] || n;
};

var Styled = styled('div')(themed('div'), space);
var components = {};
tags.forEach(function (tag) {
  components[tag] = styled(alias(tag))(themed(tag), space);
  Styled[tag] = components[tag];
});

var Context = createContext({
  theme: {},
  components: components
});

var createComponents = function createComponents() {
  var components$$1 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var next = {};
  Object.keys(components$$1).forEach(function (key) {
    next[key] = styled(components$$1[key])(themed(key));
  });
  return next;
};

var ThemeProvider$1 = function ThemeProvider$$1(_ref) {
  var theme = _ref.theme,
      components$$1 = _ref.components,
      props = _objectWithoutProperties(_ref, ["theme", "components"]);

  var outer = useContext(Context);
  var context = merge({}, outer, {
    theme: theme,
    components: createComponents(components$$1)
  });
  return jsx$1(ThemeProvider, {
    theme: context.theme
  }, jsx$1(MDXProvider, {
    components: context.components
  }, jsx$1(Context.Provider, {
    value: context,
    children: props.children
  })));
};

var Box = styled('div')(css({
  boxSizing: 'border-box',
  minWidth: 0
}), width, space, color, flex);
var Flex = styled(Box)({
  display: 'flex'
}, flexWrap, alignItems, justifyContent, flexDirection); // root/page layout

var Layout = function Layout(props) {
  return jsx$1(Box, _objectSpread({}, props, {
    css: {
      minHeight: '100vh',
      display: 'flex',
      flexDirection: 'column'
    }
  }));
};
var Header = function Header(props) {
  return jsx$1(Box, _objectSpread({
    as: 'header'
  }, props, {
    css: {
      display: 'flex'
    }
  }));
};
var Main = function Main(props) {
  return jsx$1(Box, _objectSpread({}, props, {
    css: {
      flex: '1 1 auto'
    }
  }));
};
var Container = function Container(props) {
  return jsx$1(Box, _objectSpread({}, props, {
    css: {
      width: '100%',
      minWidth: 0,
      maxWidth: 1024,
      mx: 'auto',
      p: 4
    }
  }));
};
var Footer = function Footer(props) {
  return jsx$1(Box, _objectSpread({
    as: 'footer'
  }, props, {
    css: {
      display: 'flex'
    }
  }));
};

export { jsx$1 as jsx, Context, ThemeProvider$1 as ThemeProvider, Styled, Box, Flex, Layout, Header, Main, Container, Footer, ThemeProvider$1 as ComponentProvider };
//# sourceMappingURL=index.esm.js.map
